@@ -25,12 +25,13 @@ from datetime import datetime
 from datetime import date
 from datetime import timedelta
 from datetime import time
-import decimal
 import re
 
 
 from zope.interface import classImplements
+from zope.interface import classImplementsFirst
 from zope.interface import implementer
+from zope.interface import implementedBy
 from zope.interface.interfaces import IInterface
 
 
@@ -65,9 +66,12 @@ from zope.schema.interfaces import IMinMaxLen
 from zope.schema.interfaces import IMapping
 from zope.schema.interfaces import IMutableMapping
 from zope.schema.interfaces import IMutableSequence
+from zope.schema.interfaces import INativeString
+from zope.schema.interfaces import INativeStringLine
 from zope.schema.interfaces import IObject
 from zope.schema.interfaces import INumber
 from zope.schema.interfaces import IPassword
+from zope.schema.interfaces import IPythonIdentifier
 from zope.schema.interfaces import IReal
 from zope.schema.interfaces import IRational
 from zope.schema.interfaces import ISet
@@ -103,6 +107,8 @@ from zope.schema._bootstrapfields import Bool
 from zope.schema._bootstrapfields import Int
 from zope.schema._bootstrapfields import Integral
 from zope.schema._bootstrapfields import Number
+from zope.schema._bootstrapfields import InvalidDecimalLiteral  # reimport
+from zope.schema._bootstrapfields import Decimal
 from zope.schema._bootstrapfields import Password
 from zope.schema._bootstrapfields import Rational
 from zope.schema._bootstrapfields import Real
@@ -131,23 +137,43 @@ classImplements(Field, IField)
 MinMaxLen.min_length = FieldProperty(IMinMaxLen['min_length'])
 MinMaxLen.max_length = FieldProperty(IMinMaxLen['max_length'])
 
-classImplements(Text, IText)
-classImplements(TextLine, ITextLine)
-classImplements(Password, IPassword)
-classImplements(Bool, IBool)
-classImplements(Iterable, IIterable)
-classImplements(Container, IContainer)
+classImplementsFirst(Text, IText)
+classImplementsFirst(TextLine, ITextLine)
+classImplementsFirst(Password, IPassword)
+classImplementsFirst(Bool, IBool)
+classImplementsFirst(Iterable, IIterable)
+classImplementsFirst(Container, IContainer)
 
-classImplements(Number, INumber)
-classImplements(Complex, IComplex)
-classImplements(Real, IReal)
-classImplements(Rational, IRational)
-classImplements(Integral, IIntegral)
-classImplements(Int, IInt)
+classImplementsFirst(Number, INumber)
+classImplementsFirst(Complex, IComplex)
+classImplementsFirst(Real, IReal)
+classImplementsFirst(Rational, IRational)
+classImplementsFirst(Integral, IIntegral)
+classImplementsFirst(Int, IInt)
+classImplementsFirst(Decimal, IDecimal)
 
-classImplements(Object, IObject)
+classImplementsFirst(Object, IObject)
 
 
+class implementer_if_needed(object):
+    # Helper to make sure we don't redundantly implement
+    # interfaces already inherited. Doing so tends to produce
+    # problems with the C3 order. This is used when we cannot
+    # statically determine if we need the interface or not, e.g,
+    # because we're picking different base classes under some circumstances.
+    def __init__(self, *ifaces):
+        self._ifaces = ifaces
+
+    def __call__(self, cls):
+        ifaces_needed = []
+        implemented = implementedBy(cls)
+        ifaces_needed = [
+            iface
+            for iface in self._ifaces
+            if not implemented.isOrExtends(iface)
+        ]
+        return implementer(*ifaces_needed)(cls)
+
 
 @implementer(ISourceText)
 class SourceText(Text):
@@ -170,11 +196,27 @@ class Bytes(MinMaxLen, Field):
         self.validate(value)
         return value
 
-# for things which are of the str type on both Python 2 and 3
-if PY3:  # pragma: no cover
-    NativeString = Text
-else:  # pragma: no cover
-    NativeString = Bytes
+
+@implementer_if_needed(INativeString, IFromUnicode, IFromBytes)
+class NativeString(Text if PY3 else Bytes):
+    """
+    A native string is always the type `str`.
+
+    In addition to :class:`~zope.schema.interfaces.INativeString`,
+    this implements :class:`~zope.schema.interfaces.IFromUnicode` and
+    :class:`~zope.schema.interfaces.IFromBytes`.
+
+    .. versionchanged:: 4.9.0
+       This is now a distinct type instead of an alias for either `Text` or `Bytes`,
+       depending on the platform.
+    """
+    _type = str
+
+    if PY3:
+        def fromBytes(self, value):
+            value = value.decode('utf-8')
+            self.validate(value)
+            return value
 
 
 @implementer(IASCII)
@@ -191,17 +233,34 @@ class ASCII(NativeString):
 
 @implementer(IBytesLine)
 class BytesLine(Bytes):
-    """A Text field with no newlines."""
+    """A `Bytes` field with no newlines."""
 
     def constraint(self, value):
         # TODO: we should probably use a more general definition of newlines
         return b'\n' not in value
 
-# for things which are of the str type on both Python 2 and 3
-if PY3:  # pragma: no cover
-    NativeStringLine = TextLine
-else:  # pragma: no cover
-    NativeStringLine = BytesLine
+
+@implementer_if_needed(INativeStringLine, IFromUnicode, IFromBytes)
+class NativeStringLine(TextLine if PY3 else BytesLine):
+    """
+    A native string is always the type `str`; this field excludes
+    newlines.
+
+    In addition to :class:`~zope.schema.interfaces.INativeStringLine`,
+    this implements :class:`~zope.schema.interfaces.IFromUnicode` and
+    :class:`~zope.schema.interfaces.IFromBytes`.
+
+    .. versionchanged:: 4.9.0
+       This is now a distinct type instead of an alias for either `TextLine`
+       or `BytesLine`, depending on the platform.
+    """
+    _type = str
+
+    if PY3:
+        def fromBytes(self, value):
+            value = value.decode('utf-8')
+            self.validate(value)
+            return value
 
 
 @implementer(IASCIILine)
@@ -279,70 +338,6 @@ class Float(Real):
     _validation_error = InvalidFloatLiteral
 
 
-class InvalidDecimalLiteral(ValueError, ValidationError):
-    "Raised by decimal fields"
-
-
-@implementer(IDecimal)
-class Decimal(Number):
-    """
-    A field representing a native :class:`decimal.Decimal` and implementing
-    :class:`zope.schema.interfaces.IDecimal`.
-
-    The :meth:`fromUnicode` method only accepts values that can be parsed
-    by the ``Decimal`` constructor::
-
-        >>> from zope.schema._field import Decimal
-        >>> f = Decimal()
-        >>> f.fromUnicode("1")
-        Decimal('1')
-        >>> f.fromUnicode("125.6")
-        Decimal('125.6')
-        >>> f.fromUnicode("1+0j") # doctest: +IGNORE_EXCEPTION_DETAIL
-        Traceback (most recent call last):
-        ...
-        InvalidDecimalLiteral: Invalid literal for Decimal(): 1+0j
-        >>> f.fromUnicode("1/2") # doctest: +IGNORE_EXCEPTION_DETAIL
-        Traceback (most recent call last):
-        ...
-        InvalidDecimalLiteral: Invalid literal for Decimal(): 1/2
-        >>> f.fromUnicode(str(2**31234) + '.' + str(2**256)) # doctest: +ELLIPSIS
-        Decimal('2349...936')
-        >>> f.fromUnicode("not a number") # doctest: +IGNORE_EXCEPTION_DETAIL
-        Traceback (most recent call last):
-        ...
-        InvalidDecimalLiteral: could not convert string to float: not a number
-
-    Likewise for :meth:`fromBytes`::
-
-        >>> from zope.schema._field import Decimal
-        >>> f = Decimal()
-        >>> f.fromBytes(b"1")
-        Decimal('1')
-        >>> f.fromBytes(b"125.6")
-        Decimal('125.6')
-        >>> f.fromBytes(b"1+0j") # doctest: +IGNORE_EXCEPTION_DETAIL
-        Traceback (most recent call last):
-        ...
-        InvalidDecimalLiteral: Invalid literal for Decimal(): 1+0j
-        >>> f.fromBytes(b"1/2") # doctest: +IGNORE_EXCEPTION_DETAIL
-        Traceback (most recent call last):
-        ...
-        InvalidDecimalLiteral: Invalid literal for Decimal(): 1/2
-        >>> f.fromBytes((str(2**31234) + '.' + str(2**256)).encode("ascii")) # doctest: +ELLIPSIS
-        Decimal('2349...936')
-        >>> f.fromBytes(b"not a number") # doctest: +IGNORE_EXCEPTION_DETAIL
-        Traceback (most recent call last):
-        ...
-        InvalidDecimalLiteral: could not convert string to float: not a number
-
-
-    """
-    _type = decimal.Decimal
-    _unicode_converters = (decimal.Decimal,)
-    _validation_error = InvalidDecimalLiteral
-
-
 @implementer(IDatetime)
 class Datetime(Orderable, Field):
     __doc__ = IDatetime.__doc__
@@ -488,7 +483,10 @@ class Choice(Field):
             raise ConstraintNotSatisfied(value, self.__name__).with_field_and_value(self, value)
 
 
-@implementer(IFromUnicode, IFromBytes)
+# Both of these are inherited from the parent; re-declaring them
+# here messes with the __sro__ of subclasses, causing them to be
+# inconsistent with C3.
+# @implementer(IFromUnicode, IFromBytes)
 class _StrippedNativeStringLine(NativeStringLine):
 
     _invalid_exc_type = None
@@ -544,11 +542,48 @@ class URI(_StrippedNativeStringLine):
         raise InvalidURI(value).with_field_and_value(self, value)
 
 
+# An identifier is a letter or underscore, followed by
+# any number of letters, underscores, and digits.
+_identifier_pattern = r'[a-zA-Z_]+\w*'
+
+# The whole string must match to be an identifier
+_is_identifier = re.compile('^' + _identifier_pattern + '$').match
+
 _isdotted = re.compile(
-    r"([a-zA-Z][a-zA-Z0-9_]*)"
-    r"([.][a-zA-Z][a-zA-Z0-9_]*)*"
-    # use the whole line
-    r"$").match
+    # The start of the line, followed by an identifier,
+    '^' + _identifier_pattern
+    # optionally followed by .identifier any number of times
+    + r"([.]" + _identifier_pattern + r")*"
+    # followed by the end of the line.
+    + r"$").match
+
+
+@implementer(IPythonIdentifier)
+class PythonIdentifier(_StrippedNativeStringLine):
+    """
+    This field describes a python identifier, i.e. a variable name.
+
+    Empty strings are allowed.
+
+    Identifiers can be validated from both unicode values and bytes values,
+    producing a native text string in both cases::
+
+        >>> from zope.schema import PythonIdentifier
+        >>> field = PythonIdentifier()
+        >>> field.fromUnicode(u'zope')
+        'zope'
+        >>> field.fromBytes(b'_zope')
+        '_zope'
+        >>> field.fromUnicode(u'   ')
+        ''
+
+    .. versionadded:: 4.9.0
+    """
+
+    def _validate(self, value):
+        super(PythonIdentifier, self)._validate(value)
+        if value and not _is_identifier(value):
+            raise InvalidValue(value).with_field_and_value(self, value)
 
 
 @implementer(IDottedName)
@@ -566,10 +601,13 @@ class DottedName(_StrippedNativeStringLine):
         'zope.schema'
         >>> field.fromBytes(b'zope.schema')
         'zope.schema'
+        >>> field.fromUnicode(u'zope._schema')
+        'zope._schema'
 
     .. versionchanged:: 4.8.0
         Implement :class:`zope.schema.interfaces.IFromBytes`
-
+    .. versionchanged:: 4.9.0
+        Allow leading underscores in each component.
     """
 
     _invalid_exc_type = InvalidDottedName
